﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Security.Cryptography;
using System.ComponentModel.DataAnnotations;

using PersonalFinance.Utils;
using System.ComponentModel.DataAnnotations.Schema;
using System.Globalization;
using PersonalFinance.Entity;
using System.Collections.ObjectModel;

namespace PersonalFinance
{
    /// <summary>
    /// User class for storing payments of each member of the family
    /// Each user has its own financial management
    /// </summary>
    public class User
    {
        public User() {
            payments = new ObservableCollection<Payment>();
            scheduledPayments = new ObservableCollection<ScheduledPayment>();  
        }

        public User(string password)
        {
            SetPassword(password);
            payments = new ObservableCollection<Payment>();
            scheduledPayments = new ObservableCollection<ScheduledPayment>();  
        }

        /// <summary>
        /// Unique user identification number (Primary key)
        /// </summary>
        [Key]
        public int UserId { get; set; }

        /// <summary>
        /// Unique name for sign in
        /// </summary>
        private string login;
        [Required]
        public string Login {
            get { return login; }
            set
            {
                if (value == null) { throw new ArgumentNullException("Login cannot be null."); }
                else if (value.Length < 3) { throw new ArgumentOutOfRangeException("Login must be at least 3 character length!"); }
                else { login = RemoveDiacritics(value); }
            } 
        }

        private string RemoveDiacritics(string text)
        {
            var normalizedString = text.Normalize(NormalizationForm.FormD);
            var stringBuilder = new StringBuilder();

            foreach (var c in normalizedString)
            {
                var unicodeCategory = CharUnicodeInfo.GetUnicodeCategory(c);
                if (unicodeCategory != UnicodeCategory.NonSpacingMark)
                {
                    stringBuilder.Append(c);
                }
            }

            return stringBuilder.ToString().Normalize(NormalizationForm.FormC);
        }

        /// <summary>
        /// Password for sign in
        /// </summary>
        [Required]
        private string password;
        public string Password { get { return password; } }

        private string name;
        /// <summary>
        /// User real name
        /// </summary>
        public string Name
        {
            get { return name; }
            set
            {
                if (value == null) { throw new ArgumentNullException("Name cannot be null."); }
                else { name = value; }
            }
        }

        /// <summary>
        /// User real lastname
        /// </summary>
        private string lastname;
        public string Lastname
        {
            get { return lastname; }
            set
            {
                if (value == null) { throw new ArgumentNullException("Lastname cannot be null."); }
                else { lastname = value; }
            }
        }

        /// <summary>
        /// Sex (gender) of user
        /// </summary>
        public Sex Sex { get; set; }

        public void SetPassword(string passwordToHash)
        {
            if (passwordToHash == null) throw new ArgumentNullException();
            else if (passwordToHash.Length < 6) throw new ArgumentOutOfRangeException("Password must be at least 6 character length!");

            this.password = PasswordHash.CreateHash(passwordToHash);
        }

        public bool ComparePassword(string passwordToCompare)
        {
            return PasswordHash.ValidatePassword(passwordToCompare, Password);
        }

        private ObservableCollection<Payment> payments;
        public virtual ObservableCollection<Payment> Payments
        {
            get { return payments; }
            set { payments = value; }
        }

        public void AddPayment(Payment p)
        {
            this.payments.Add(p);
        }

        private ObservableCollection<ScheduledPayment> scheduledPayments;
        public virtual ObservableCollection<ScheduledPayment> ScheduledPayments
        {
            get { return scheduledPayments; }
            set { scheduledPayments = value; }
        }

        public void AddScheduledPayment(ScheduledPayment p)
        {
            this.scheduledPayments.Add(p);
        }

        public override string ToString()
        {
            return string.Format("User {0}: {1} ({2} {3}), Payments: {4}, Scheduled: {5}",
                UserId, Login, Name, Lastname, payments.Count, scheduledPayments.Count);
        }
    }
}
